#
# ZAxisVideo : Zope Axis Video Servers Controller
#
# (c) 2003 Jerome Alet - <alet@unice.fr>
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#

import urllib

from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import Implicit
from Globals import DTMLFile, Persistent, InitializeClass, MessageDialog
from AccessControl.Role import RoleManager
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

manage_addZAxisVideoForm = DTMLFile('dtml/ZAxisVideoAdd', globals())

version = "1.01"

def manage_addZAxisVideo(self, id, title='', location='', ptzcams=[], nonptzcams=[], width=176, height=144, zoomstep=1250, REQUEST=None) :
    """Adds the ZAxisVideo object in the current Folder."""
    ref = ZAxisVideo(id, title, location, ptzcams, nonptzcams, width, height, zoomstep)
    dest = self.Destination()
    dest._setObject(id, ref)
    if REQUEST is not None:
        try:    url=self.DestinationURL()
        except: url=REQUEST['URL1']
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)
    return id

class ZAxisVideo(Implicit, Persistent, SimpleItem, PropertyManager) :
    """ZAxisVideo implementation."""

    _properties = (
                      {'id':'title', 'type': 'string', 'mode':'w'},
                      {'id':'location', 'type': 'string', 'mode':'w'},
                      {'id':'ptzcams', 'type': 'lines', 'mode':'w'},
                      {'id':'nonptzcams', 'type': 'lines', 'mode':'w'},
                      {'id':'width', 'type': 'int', 'mode':'w'},
                      {'id':'height', 'type': 'int', 'mode':'w'},
                      {'id':'zoomstep', 'type': 'int', 'mode':'w'},
                  )

    meta_type = "ZAxisVideo"
    
    security = ClassSecurityInfo()
    security.declareObjectProtected('View')
    
    def __init__(self, id, title = '', location='', ptzcams=[], nonptzcams=[], width=176, height=144, zoomstep=1250) :
        """Initialization."""
        self.id = id
        self.title = title.strip()
        self.location = location.strip()    
        self.ptzcams = ptzcams
        self.nonptzcams = nonptzcams
        self.width = int(width)
        self.height = int(height)
        self.zoomstep = int(zoomstep)

    manage_options = PropertyManager.manage_options + ( 
                       { "label" : "View", "action" : "index_html" },
                     ) + SimpleItem.manage_options

    security.declareProtected('View', 'index_html')
    index_html = DTMLFile("dtml/ZAxisVideoView", globals())
    security.declareProtected('View', '__call__')
    __call__ = index_html
    
    security.declareProtected('View', 'cleanCamList')
    def cleanCamList(self, cameras) :
        """Cleans a camera list : remove duplicates, invalid or empty elements, return the result sorted by numeric ids."""
        newcameras = []
        for camera in cameras :
            try :
                camid = int(camera.strip())
            except (ValueError, AttributeError) :    
                pass
            else :    
                camstr = str(camid)
                if camstr not in newcameras :
                    newcameras.append(camstr)            
        newcameras.sort(lambda x, y : cmp(int(x), int(y)))
        return newcameras
        
    security.declareProtected('View', 'getPTZCameras')
    def getPTZCameras(self) :
        """Returns a list of the Pan/Tilt/Zoom cameras, cleaned."""
        return self.cleanCamList(self.ptzcams)
        
    security.declareProtected('View', 'getNonPTZCameras')
    def getNonPTZCameras(self) :    
        """Returns a list of the Non-Pan/Tilt/Zoom cameras, cleaned."""
        ptz = self.cleanCamList(self.ptzcams)
        nonptz = []
        for camera in self.cleanCamList(self.nonptzcams) :
            if not camera in ptz :
                nonptz.append(camera)
        return nonptz        
        
    security.declareProtected('View', 'getCameraURL')
    def getCameraURL(self, camera) :    
        """Returns the URL to a camera picture."""
        location = self.location 
        if not location.endswith('/') :
            location += '/'
        return "%saxis-cgi/mjpg/video.cgi?camera=%s&resolution=%ix%i" % (location, str(camera), self.width, self.height)
        
    security.declareProtected('View', 'getPTZIconURL')
    def getPTZIconURL(self, icon) :    
        """Returns the URL of a Pan/Tilt/Zoom icon."""
        return "%s/misc_/%s/%s" % (self.absolute_url(), self.__class__.__name__, icon)
        
    security.declareProtected('Control ZAxisVideo instances', 'moveLeft')    
    def moveLeft(self, camera, REQUEST=None) :
        """Moves a camera to the left."""
        self._cameraAction(camera, move="left")
        self._doRedirect(REQUEST)
        
    security.declareProtected('Control ZAxisVideo instances', 'moveRight')    
    def moveRight(self, camera, REQUEST=None) :
        """Moves a camera to the right."""
        self._cameraAction(camera, move="right")
        self._doRedirect(REQUEST)
        
    security.declareProtected('Control ZAxisVideo instances', 'moveUp')    
    def moveUp(self, camera, REQUEST=None) :
        """Moves a camera up."""
        self._cameraAction(camera, move="up")
        self._doRedirect(REQUEST)
        
    security.declareProtected('Control ZAxisVideo instances', 'moveDown')    
    def moveDown(self, camera, REQUEST=None) :
        """Moves a camera down."""
        self._cameraAction(camera, move="down")
        self._doRedirect(REQUEST)
        
    security.declareProtected('Control ZAxisVideo instances', 'moveHome')    
    def moveHome(self, camera, REQUEST=None) :
        """Moves a camera back home."""
        self._cameraAction(camera, move="home")
        self._doRedirect(REQUEST)
        
    security.declareProtected('Control ZAxisVideo instances', 'zoomIn')    
    def zoomIn(self, camera, REQUEST=None) :
        """Zooms a camera in."""
        self._cameraAction(camera, rzoom="%i" % self.zoomstep)
        self._doRedirect(REQUEST)
        
    security.declareProtected('Control ZAxisVideo instances', 'zoomOut')    
    def zoomOut(self, camera, REQUEST=None) :
        """Zooms a camera out."""
        self._cameraAction(camera, rzoom="-%i" % self.zoomstep)
        self._doRedirect(REQUEST)
        
    def _doRedirect(self, REQUEST) :    
        """Redirects the browser to where we come from if needed."""
        if REQUEST is not None :
            REQUEST.RESPONSE.redirect(REQUEST["HTTP_REFERER"])
            
    def _cameraAction(self, camera, **kw) :
        """Tells a camera to move."""
        location = self.location 
        if not location.endswith('/') :
            location += '/'
        args = ["%s=%s" % (k, v) for (k, v) in kw.items()]
        url = "&".join(["%saxis-cgi/com/ptz.cgi?camera=%s" % (location, camera)] + args)
        try :
            action = urllib.urlopen(url)
        except IOError :    
            # video server down maybe
            pass
        else :    
            dummy = action.read() # just in case we would need to examine the result.
            action.close()
        
InitializeClass(ZAxisVideo)
